@@ -15,7 +15,7 @@
  * Modifications and updates by: Jesse Smith
  * Date: May 4, 2011
  * Version 1.2 
- * Date: Jam 29, 2013
+ * Date: Jan 29, 2013
  * Version 1.2 and newer
  *
  */
@@ -77,7 +77,7 @@
 #endif
 
 #ifndef VERSION
-#define VERSION 1.9
+#define VERSION 2.0
 #endif
 
 //pid of the controlled process
@@ -485,7 +485,7 @@ void increase_priority()
 
 void print_usage(FILE *stream,int exit_code) {
         fprintf(stream, "CPUlimit version %1.1f\n", VERSION);
-	fprintf(stream, "Usage: %s TARGET [OPTIONS...] [PROGRAM]\n",program_name);
+	fprintf(stream, "Usage: %s TARGET [OPTIONS...] [-- PROGRAM]\n",program_name);
 	fprintf(stream, "   TARGET must be exactly one of these:\n");
 	fprintf(stream, "      -p, --pid=N        pid of the process\n");
 	fprintf(stream, "      -e, --exe=FILE     name of the executable program file\n");
@@ -507,6 +507,8 @@ void print_usage(FILE *stream,int exit_code) {
 	fprintf(stream, "      -v, --verbose      show control statistics\n");
 	fprintf(stream, "      -z, --lazy         exit if there is no suitable target process,\n");
         fprintf(stream, "                         or if it dies\n");
+        fprintf(stream, "          --             This is the final CPUlimit option. All following\n");
+        fprintf(stream, "                         options are for another program we will launch.\n");
 	fprintf(stream, "      -h, --help         display this help and exit\n");
 	exit(exit_code);
 }
@@ -564,6 +566,7 @@ int main(int argc, char **argv) {
 
         NCPU = get_ncpu();
 
+        opterr = 0;      // avoid unwanted error messages for unknown parameters
 	do {
 		next_option = getopt_long (argc, argv, short_options,long_options, NULL);
 		switch(next_option) {
@@ -573,8 +576,11 @@ int main(int argc, char **argv) {
                                 break;
 			case 'p':
 				pid=atoi(optarg);
-				pid_ok = TRUE;
-                                lazy = TRUE;
+                                if (pid)   // valid PID
+                                {
+				  pid_ok = TRUE;
+                                  lazy = TRUE;
+                                }
                                 last_known_argument += 2;
 				break;
 			case 'e':
@@ -617,6 +623,10 @@ int main(int argc, char **argv) {
 				print_usage (stdout, 1);
                                 last_known_argument++;
 				break;
+                        case 'o':
+                                last_known_argument++;
+                                next_option = -1;
+                                break;
 			case '?':
 				print_usage (stderr, 1);
                                 last_known_argument++;
@@ -629,17 +639,22 @@ int main(int argc, char **argv) {
 	} while(next_option != -1);
 
 
-        if (last_known_argument + 1 < argc)
+        // try to launch a program passed on the command line
+        // But only if we do not already have a PID to watch
+        if ( (last_known_argument + 1 < argc) && (pid_ok == FALSE) )
         {
            last_known_argument++;
+           // if we stopped on "--" jump to the next parameter
+           if ( (last_known_argument + 1 < argc) && (! strcmp(argv[last_known_argument], "--") ) )
+               last_known_argument++;
            pid_t forked_pid;
            // try to launch remaining arguments
            if (verbose)
            {
                int index = last_known_argument;
-               printf("Launching %s ", argv[index]);
+               printf("Launching %s", argv[index]);
                for (index = last_known_argument + 1; index < argc; index++)
-                    printf("%s", argv[index]);
+                    printf(" %s", argv[index]);
                printf(" with limit %d\n", perclimit);
            }
            forked_pid = fork();
